// Note: run `npm install` first!

var regenerate = require('./regenerate');
var template = require('lodash.template');

// Which Unicode version should be used?
var version = '6.3.0';

// Shorthand function
var get = function(what) {
	return require('unicode-' + version + '/' + what + '/code-points');
};

// Unicode categories needed to construct the ES5 regex
var Lu = get('categories/Lu');
var Ll = get('categories/Ll');
var Lt = get('categories/Lt');
var Lm = get('categories/Lm');
var Lo = get('categories/Lo');
var Nl = get('categories/Nl');
var Mn = get('categories/Mn');
var Mc = get('categories/Mc');
var Nd = get('categories/Nd');
var Pc = get('categories/Pc');

// Unicode categories needed to construct the ES6 regex
var Pattern_Syntax = get('properties/Pattern_Syntax');
var Pattern_White_Space = get('properties/Pattern_White_Space');

var compileRegex = template('identifierStart:\n\n' +
	'<%= identifierStart %>\n\nidentifierPart:\n\n<%= identifierPart %>\n\n\n\n');

var generateES5Regex = function() { // ES 5.1
	// http://mathiasbynens.be/notes/javascript-identifiers#valid-identifier-names
	var identifierStart = regenerate('$', '_')
		.add(Lu, Ll, Lt, Lm, Lo, Nl)
		.removeRange(0x010000, 0x10FFFF); // remove astral symbols
	var identifierPart = identifierStart.clone()
		.add('\u200C', '\u200D', Mn, Mc, Nd, Pc)
		.removeRange(0x010000, 0x10FFFF); // remove astral symbols


	var regex = compileRegex({
		'identifierStart': identifierStart.toString(),
		'identifierPart': identifierPart.toString()
	});
	return regex;
};

var generateES6Regex = function() {
	// http://people.mozilla.org/~jorendorff/es6-draft.html#sec-7.6
	// http://www.unicode.org/reports/tr31/#Default_Identifier_Syntax
	var identifierStart = regenerate()
		.add(Lu, Ll, Lt, Lm, Lo, Nl)
		.remove(Pattern_Syntax, Pattern_White_Space)
		.add('$', '_');
		// Note: don’t remove astral symbols, as per
		// https://mail.mozilla.org/pipermail/es-discuss/2012-February/020762.html
	var identifierPart = identifierStart.clone()
		.add(Mn, Mc, Nd, Pc)
		.remove(Pattern_Syntax, Pattern_White_Space)
		.add('$', '_', '\u200C', '\u200D');
		// Note: don’t remove astral symbols, as per
		// https://mail.mozilla.org/pipermail/es-discuss/2012-February/020762.html

	var regex = compileRegex({
		'identifierStart': identifierStart.toString(),
		'identifierPart': identifierPart.toString()
	});
	return regex;
};

console.log('// ECMAScript 5.1:\n')
console.log(generateES5Regex());
console.log('\n// ECMAScript 6:\n');
console.log(generateES6Regex());
